import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.geom.Line2D;
import java.util.ArrayList;

/**
 * 
 * Clase que permite crear un componente que desplegue gráficos
 * para monitorear distintos atributos en los tanques:
 * Nivel del tanque, Apertura de Valvula, Altura de Referencia
 * 
 * @author Joaquín Moreira y Javier Nuñez
 */
public class Grafico extends JComponent implements ActionListener{
    private boolean dibujarNivel;
    private boolean dibujarValvula;
    private boolean dibujarRef;
    private int origenX;
    private int origenY;
    private final int largoEjeX=250;
    private final int largoEjeY=160;
    private Point posicion;
    private ArrayList<Point> puntos,puntosValvula;
    private ArrayList<Point> puntosRef;
    private ArrayList<Point> puntosNivel;
    
    private long timeInicio;
    private int delay=500;
    private Timer timer;
    private Tank tanque;
    private Controller control;
    
    /**
     * Constructor usado en los gráficos que requieren dibujar la altura
     * de referencia
     * 
     * @param posicion Posicion base sobre la cual se dibujara el Grafico
     * @param tanque Tanque sobre se construirá el gráfico
     * @param control En caso de que se requiere dibujar la altura de referencia
     * se entrega como parámetro la instancia de Controller
     */
    public Grafico(Point posicion,Tank tanque, Controller control){
        this(posicion,tanque);
        this.control=control;
    }
    
    /**
     * Constructor usado en los gráficos que no requieren dibujar la
     * altura de referencia.
     * 
     * @param posicion Posicion base sobre la cual se dibujara el Grafico
     * @param tanque Tanque sobre se construirá el gráfico
     */
    public Grafico(Point posicion, Tank tanque){
        this.posicion=posicion;
        this.tanque=tanque;
    }
    
    /**
     * Parámetro que responde al evento desatado por el Timer y que va
     * añadiendo puntos a las curvas respectivas
     * @param arg0 No tiene utilidad en este caso
     */
    public void actionPerformed(ActionEvent arg0) {
        
        //IF controla si se acabo el tiempo máximo para seguir dibujando gráfico
        if((System.currentTimeMillis()-timeInicio)<=20100){
            
            //Si se requiere dibujar curva del Nivel de Tanque
            if(dibujarNivel){
                double x,y;
                y=posicion.getY()+largoEjeY-(tanque.getLevel()/Tank.H)*largoEjeY;
                x=posicion.getX()+(System.currentTimeMillis()-timeInicio)*largoEjeX/20000;
                puntosNivel.add(new Point((int)x,(int)y));
                repaint();
            }
            
            //Si se requiere dibujar curva de Valvula
            if(dibujarValvula){
                double x,y;
                y=posicion.getY()+largoEjeY-(tanque.getOutValve())*largoEjeY;
                x=posicion.getX()+(System.currentTimeMillis()-timeInicio)*largoEjeX/20000;
                puntosValvula.add(new Point((int)x,(int)y));
                repaint();
            }
            
            //Si se requiere dibujar curva de Referencia
            if(dibujarRef){
                double x,y;
                y=posicion.getY()+largoEjeY-(control.getRef()/Tank.H)*largoEjeY;
                x=posicion.getX()+(System.currentTimeMillis()-timeInicio)*largoEjeX/20000;
                puntosRef.add(new Point((int)x,(int)y));
                repaint();
            }
        }
    }
    
    /**
     * Método que se encarga del dibujado del gráfico en la pantalla.
     * @param g Objeto que permite ir dibujando sobre el componente
     */
    @Override
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        Graphics2D g2=(Graphics2D)g;
        
        //Pintar Rotulo Grafico
        g2.drawString("Tanque "+tanque.getId(),(int)(posicion.getX()+(largoEjeX-5)/2),
                (int)(posicion.getY()));
        
        //Pintar Ejes
        g2.drawLine((int)posicion.getX(),(int)posicion.getY(),
                (int)posicion.getX(),(int)posicion.getY()+largoEjeY+10); //EjeY
        g2.drawLine((int)posicion.getX()-10,largoEjeY+(int)posicion.getY(),
                (int)(largoEjeX+posicion.getX()),largoEjeY+(int)posicion.getY());//EjeX
        
        //Pintar Valores Ejes
        g2.drawString("0",(int)posicion.getX()-10,largoEjeY+22);
        g2.drawString("100%",(int)posicion.getX()-10,(int)posicion.getY());
        g2.drawString("20",(int)posicion.getX()+largoEjeX-10, largoEjeY+22);
        g2.drawString("t[s]",(int)posicion.getX()+largoEjeX+3, largoEjeY+3);
        
        //Dibujar Curvas
        if(dibujarNivel){
            g2.setColor(Color.BLUE);
            for(int i=0;i<puntosNivel.size()-1;i++){
                g2.draw(new Line2D.Double(puntosNivel.get(i),puntosNivel.get(i+1)));
            }
        }
        
        if(dibujarRef){
            g2.setColor(Color.ORANGE);
            for(int i=0;i<puntosRef.size()-1;i++){
                g2.draw(new Line2D.Double(puntosRef.get(i),puntosRef.get(i+1)));
            }
        }
        
        if(dibujarValvula){
            g2.setColor(Color.RED);
            for(int i=0;i<puntosValvula.size()-1;i++){
                g2.draw(new Line2D.Double(puntosValvula.get(i),puntosValvula.get(i+1)));
            }
        }
    }
    
    /**
     * Permite habilitar el dibujado de la curva de la Valvula
     */
    public void enableValvula(){
        dibujarValvula=true;
        puntosValvula=new ArrayList<Point>();
        double x,y;
        x=posicion.getX();
        y=posicion.getY()+largoEjeY-(tanque.getOutValve()*largoEjeY);
        puntosValvula.add(new Point((int)x,(int)y));
    }
    
    /**
     * Permite habilitar el dibujado de la curva del Nivel
     */
    public void enableNivel(){
        dibujarNivel=true;
        puntosNivel=new ArrayList<Point>();
        double x,y;
        x=posicion.getX();
        y=posicion.getY()+largoEjeY-(tanque.getLevel()/Tank.H)*largoEjeY;
        puntosNivel.add(new Point((int)x,(int)y));
    }
    
    /**
     * Permite habilitar el dibujado de la curva de la altura de Referencia
     */
    public void enableRef(){
        dibujarRef=true;
        puntosRef=new ArrayList<Point>();
        double x,y;
        x=posicion.getX();
        y=posicion.getY()+largoEjeY-(control.getRef()/Tank.H)*largoEjeY;
        puntosRef.add(new Point((int)x,(int)y));
    }
    
    /**
     * Inicializa los gráficos arrancando el timer
     */
    public void start(){
        timer=new Timer(delay, this);
        timeInicio=System.currentTimeMillis();
        timer.start();
    }
}